/*
fluxy. A Realtime 2d Path Tracer
---
Copyright (C) 2008  Sebastian Mach (*1983)
seb@greenhybrid.net | http://greenhybrid.net
---
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef INTERSECTION_H
#define INTERSECTION_H

#include "material.h"
#include "vector.h"

class Intersection{
    public:
        const Material material;
        const Vector normal;
        const float distance;
        Intersection( Material material, Vector normal, float distance )
        : material(material), normal(normal), distance(distance)
        {}

        Intersection & operator = ( const Intersection &i ) {
            const_cast<Material&>(material) = i.material;
            const_cast<Vector&>(normal)     = i.normal;
            const_cast<float&>(distance)    = i.distance;
            return *this;
        }

        void calculateReflectionDirection( float &d_x, float &d_y, float in_x, float in_y ){
            const double cosI = -( normal.x*in_x + normal.y*in_y );
            d_x = in_x + 2.0f * cosI * normal.x;
            d_y = in_y + 2.0f * cosI * normal.y;
        }
};

#endif
