/*
fluxy. A Realtime 2d Path Tracer
---
Copyright (C) 2008  Sebastian Mach (*1983)
seb@greenhybrid.net | http://greenhybrid.net
---
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <math.h>
#include "ptintegrator.h"
#include "intersection.h"

/*template <class T> ColorFRGB PTIntegrator<T>::operator () ( float u, float v ) const{
    return ColorFRGB(0.5,0.5,0.5);
};
*/

template <class T> ColorFRGB PTIntegrator<T>::integrate( float u, float v ) const{
    // transform screenspace to worldspace
    const float start_x = u-0.5;
    const float start_y = v-0.5;
    float r=0.0,g=0.0,b=0.0,alpha=0.0;
    for( unsigned int numSamples=0; numSamples<8; ++numSamples ){
        // we're in 2d, so it's easy to generate a uniform distribution
        // on the circle using cos/sin
        float x = start_x;
        float y = start_y;
        float delta = ( static_cast<float>(rand()) / static_cast<float>(RAND_MAX) ) * 2.0f*M_PI;
        float d_x = cosf(delta);
        float d_y = sinf(delta);

        float i_r = 1.0;
        float i_g = 1.0;
        float i_b = 1.0;

        for( unsigned int length=0; length<2; ++length ){
            // calculate intersection
            const Intersection intersection = scene.intersect(x, y, d_x, d_y);
            if( intersection.distance < -0.0f )
                break;

            // let color at point of intersection contribute to pixel color
            float tmp_r, tmp_g, tmp_b;
            intersection.material.color.to_rgb( tmp_r, tmp_g, tmp_b );

            i_r *= tmp_r;
            i_g *= tmp_g;
            i_b *= tmp_b;

            if( intersection.material.isLuminaire ){
                // we've hit a light source -> contribute to pixel
                r += i_r;
                g += i_g;
                b += i_b;
            }

            x += d_x*intersection.distance;
            y += d_y*intersection.distance;
            // generate diffuse reflection
            do{
                delta = ( static_cast<float>(rand()) / static_cast<float>(RAND_MAX) ) * 2.0f*M_PI;
                d_x = cosf(delta);
                d_y = sinf(delta);
            }while( (d_x*intersection.normal.x + d_y*intersection.normal.y) <= 0.0 );
            // epsilon "correction" for position
            x += d_x * EPSILON;
            y += d_y * EPSILON;
        }
        ++alpha;
    }
    return ColorFRGB(r/alpha, g/alpha, b/alpha);
};

